/* Geedot Toolkit - Widget set for Geedot Web Browser
 *   File: gdtoolkit.h - generic typedefs, structs, enums, etc. as well as
 *         function declarations
 *   Author: Luke Crowthers
 *   Start Date: Aug, 2000
 *   Version: 0.1
 *   License: GPL
 */

#include <X11/Xlib.h>

#define GEEDOT_FUNCTION(x) (GeedotFunction) x

#define GEEDOT_TOPLEVEL_FRAME(x) ((GeedotToplevelFrame *) x)
#define GEEDOT_CHILD_FRAME(x) ((GeedotChildFrame *) x)
#define GEEDOT_POPUP_FRAME(x) ((GeedotPopupFrame *) x)
#define GEEDOT_PIXMAP_FRAME(x) ((GeedotPopupFrame *) x)
#define GEEDOT_TEXT(x) ((GeedotText *) x)
#define GEEDOT_PIXMAP(x) ((GeedotPixmap *) x)
#define GEEDOT_BUTTON(x) ((GeedotButton *) x)
#define GEEDOT_CHECKBOX(x) ((GeedotCheckbox *) x)
#define GEEDOT_RADIO_LIST(x) ((GeedotRadioList *) x)
#define GEEDOT_RADIO_BUTTON(x) ((GeedotRadioButton *) x)
#define GEEDOT_TEXTBOX(x) ((GeedotTextbox *) x)
#define GEEDOT_SCROLLBAR(x) ((GeedotScrollbar *) x)
#define GEEDOT_COMBOBOX(x) ((GeedotCombobox *) x)

typedef void (*GeedotFunction) ();

typedef struct _GeedotArea GeedotArea;

typedef struct _GeedotEvent GeedotEvent;
typedef struct _GeedotEventList GeedotEventList;
typedef struct _GeedotEventQueue GeedotEventQueue;

typedef struct _GeedotAction GeedotAction;
typedef struct _GeedotActionList GeedotActionList;
typedef struct _GeedotWidget GeedotWidget;
typedef struct _GeedotWidgetList GeedotWidgetList;

typedef struct _GeedotToplevelFrame GeedotToplevelFrame;
typedef struct _GeedotChildFrame GeedotChildFrame;
typedef struct _GeedotPopupFrame GeedotPopupFrame;
typedef struct _GeedotPixmapFrame GeedotPixmapFrame;
typedef struct _GeedotText GeedotText;
typedef struct _GeedotPixmap GeedotPixmap;
typedef struct _GeedotButton GeedotButton;
typedef struct _GeedotCheckbox GeedotCheckbox;
typedef struct _GeedotRadioList GeedotRadioList;
typedef struct _GeedotRadioButton GeedotRadioButton;
typedef struct _GeedotTextbox GeedotTextbox;
typedef struct _GeedotScrollbar GeedotScrollbar;
typedef struct _GeedotCombobox GeedotCombobox;
typedef struct _GeedotComboboxEntryList GeedotComboboxEntryList;
typedef struct _GeedotComboboxEntry GeedotComboboxEntry;

typedef struct _GeedotDrawPackage GeedotDrawPackage;
typedef struct _GeedotColor GeedotColor;
typedef struct _GeedotDefaultColorSet GeedotDefaultColorSet;

enum {
	GEEDOT_LIGHT_PURPLE,
	GEEDOT_PURPLE,
	GEEDOT_DARK_PURPLE,
	GEEDOT_LIGHT_GREY,
	GEEDOT_GREY,
	GEEDOT_DARK_GREY
};

typedef enum {
	BUTTON1_DEPRESSED = 1 << 1,
	BUTTON2_DEPRESSED = 1 << 2,
	BUTTON3_DEPRESSED = 1 << 3
} GeedotPointerState;

typedef enum {
	GeedotButtonPress = 1 << 1,
	GeedotButtonRelease = 1 << 2,
	GeedotButtonPressRelease = 1 << 3,
	GeedotPointerMotion = 1 << 4,
	GeedotWidgetEnter = 1 << 5,
	GeedotWidgetLeave = 1 << 6,
	GeedotExpose = 1 << 7,
	GeedotKeyPress = 1 << 8,
	GeedotKeyRelease = 1 << 9,
	GeedotFrameConfigure = 1 << 10,
	GeedotFrameEnter = 1 << 11,
	GeedotFrameLeave = 1 << 12
} GeedotEventMask;

typedef enum {
	GEEDOT_ROOT,
	GEEDOT_TOPLEVEL_FRAME,
	GEEDOT_CHILD_FRAME,
	GEEDOT_POPUP_FRAME,
	GEEDOT_PIXMAP_FRAME,
	GEEDOT_TEXT,
	GEEDOT_BUTTON,
	GEEDOT_RADIO_LIST,
	GEEDOT_RADIO_BUTTON,
	GEEDOT_CHECKBOX,
	GEEDOT_TEXTBOX,
	GEEDOT_VERT_SCROLLBAR,
	GEEDOT_HORIZ_SCROLLBAR,
	GEEDOT_COMBOBOX,
	GEEDOT_COMBOBOX_ENTRY,
	GEEDOT_USER_WIDGET
} GeedotWidgetType;

typedef enum {
	GEEDOT_TEXT_BUTTON,
	GEEDOT_PIXMAP_BUTTON,
	GEEDOT_TEXT_PIXMAP_BUTTON,
	GEEDOT_PIXMAP_TEXT_BUTTON
} GeedotButtonType;

struct _GeedotArea {
	int x;
	int y;
	int width;
	int height;
};

struct _GeedotEvent {
	GeedotEventMask type;
	void *event;
};

struct _GeedotEventList {
	GeedotEvent *event;
	GeedotEventList *next;
};

struct _GeedotEventQueue {
	GeedotEventList *queue;
	int events;
};

struct _GeedotAction {
	GeedotEventMask trigger_mask;
	GeedotFunction  response;
	void           *response_data;
	GeedotArea      area;
};

struct _GeedotDrawPackage {
	GeedotFunction  redraw;
	void           *redraw_data;
};

struct _GeedotColor {
	char *value;
	XColor color;
};

struct _GeedotDefaultColorSet {
	GeedotColor light1;
	GeedotColor mid1;
	GeedotColor dark1;
	GeedotColor light2;
	GeedotColor mid2;
	GeedotColor dark2;
	GeedotColor foreground;
	GeedotColor background;
};

struct _GeedotWidget {
	Display *display;
	Drawable drawable;
	Pixmap backbuffer;
	GC frame_gc;
	XFontStruct *font;
	Colormap colormap;
	char *selection;
	GeedotPointerState *pointer_state;
	GeedotDefaultColorSet *geedot_colors;

	GeedotEventQueue event_queue;
	GeedotEventMask handled_events;
	GeedotWidget *parent;
	GeedotWidget *owner;
	GeedotWidgetType type;
	GeedotArea area;
	GeedotWidgetList *first_widget;
	GeedotWidgetList *last_widget;
	GeedotActionList *first_action;
	GeedotActionList *last_action;
	GeedotActionList *first_widget_action;
	GeedotActionList *last_widget_action;
	Bool is_active;
	Bool is_visible;
	GeedotDrawPackage draw_package;
	void *widget_data;
};

struct _GeedotWidgetList {
	GeedotWidget *widget;
	GeedotWidgetList *next;
};

struct _GeedotToplevelFrame {
	char *title;
	char *selection;
};

struct _GeedotChildFrame {
	
};

struct _GeedotPopupFrame {
	char *title;
	char *selection;
};

struct _GeedotPixmapFrame {
	
};

struct _GeedotText {
	char *text;
};

struct _GeedotPixmap {
	Pixmap pixmap;
};

struct _GeedotButton {
	GeedotButtonType type;
	char *label;
	Pixmap pixmap;
};

struct _GeedotCheckbox {
	Bool is_checked;
	char *label;
};

struct _GeedotRadioList {
	GeedotWidget *selected_button;
};

struct _GeedotRadioButton {
	GeedotRadioList *radio_list;
	char *label;
};

struct _GeedotTextbox {
	char *label;
	char *text;
	Pixmap text_pixmap;
	char *selection;
	int   cursor_start;
	int   cursor_stop;
	int   view_start;
};

struct _GeedotScrollbar {
	GeedotWidget *decrease_button;
	GeedotWidget *increase_button;
	float  percent_start;
	float  percent;
	int grab_start;
};

struct _GeedotCombobox {
	Bool locked;
	int entries_width;
	GeedotWidgetList *first_entry;
	GeedotWidgetList *last_entry;
	GeedotWidget *option;
	GeedotWidget *button;
	GeedotWidget *popup;
};

struct _GeedotComboboxEntry {
	GeedotCombobox *owner;
	char *text;
	int   priority;
};

struct _GeedotComboboxEntryList {
	GeedotComboboxEntry entry;
	GeedotComboboxEntryList *next;
};

struct _GeedotActionList {
	GeedotAction *action;
	GeedotActionList *next;
};

/* TODO:
 *  Reorganize declarations by type (draw, util, etc.)
 *  then by widget (button, checkbox, etc.)
 */

GeedotWidget *__gdot_root_widget;
GeedotWidget *gdot_get_root_widget (Display *);

GeedotWidget *gdot_add_toplevel_frame (GeedotWidget *, GeedotArea);
GeedotWidget *gdot_add_child_frame (GeedotWidget *, GeedotArea);
GeedotWidget *gdot_add_popup_frame (GeedotWidget *, GeedotArea);
GeedotWidget *gdot_add_pixmap_frame (GeedotWidget *, GeedotArea);
int gdot_register_action(GeedotWidget *, GeedotAction);
int gdot_register_widget_action(GeedotWidget *, GeedotAction);
int gdot_show_frame (GeedotWidget *);
int gdot_hide_frame (GeedotWidget *);
int gdot_move_frame (GeedotWidget *, GeedotArea);
int gdot_resize_frame (GeedotWidget *, GeedotArea);
int gdot_force_resize_frame (GeedotWidget *, GeedotArea);
int gdot_raise_frame (GeedotWidget *);

GeedotWidget *gdot_add_text (GeedotWidget *, GeedotArea, char *);
GeedotWidget *gdot_add_button (GeedotWidget *, GeedotArea);
GeedotWidget *gdot_add_text_button (GeedotWidget *, GeedotArea, char *);
GeedotWidget *gdot_add_pixmap_button (GeedotWidget *, GeedotArea, Pixmap);
GeedotWidget *gdot_add_text_pixmap_button (GeedotWidget *, GeedotArea, char *, Pixmap);
GeedotWidget *gdot_add_pixmap_text_button (GeedotWidget *, GeedotArea, Pixmap, char *);
GeedotWidget *gdot_attach_button_to_widget (GeedotWidget *);
GeedotWidget *gdot_add_checkbox (GeedotWidget *, GeedotArea);
GeedotWidget *gdot_add_radio_list (GeedotWidget *, GeedotArea);
GeedotWidget *gdot_add_radio_button (GeedotWidget *, GeedotArea);
GeedotWidget *gdot_add_textbox (GeedotWidget *, GeedotArea);
GeedotWidget *gdot_add_scrollbar (GeedotWidget *, GeedotArea);
GeedotWidget *gdot_add_combobox (GeedotWidget *, GeedotArea);
GeedotWidget *gdot_add_locked_combobox (GeedotWidget *, GeedotArea);
GeedotWidget *gdot_add_combobox_entry (GeedotWidget *, char *);

int gdot_set_fg_color (GeedotWidget *, GeedotColor);
int gdot_set_bg_color (GeedotWidget *, GeedotColor);
int gdot_draw_point (GeedotWidget *, unsigned int, unsigned int);
int gdot_draw_line (GeedotWidget *, unsigned int, unsigned int, unsigned int, unsigned int);
int gdot_draw_rect (GeedotWidget *, GeedotArea);
int gdot_fill_rect (GeedotWidget *, GeedotArea);
int gdot_draw_oval (GeedotWidget *, GeedotArea);
int gdot_fill_oval (GeedotWidget *, GeedotArea);

int gdot_draw_button_highlighted (GeedotWidget *);
int gdot_draw_button_depressed (GeedotWidget *);
int gdot_draw_button_plain (GeedotWidget *);
int gdot_draw_checkbox_highlighted (GeedotWidget *);
int gdot_draw_checkbox_depressed (GeedotWidget *);
int gdot_draw_checkbox_plain (GeedotWidget *);
int gdot_draw_radio_button_plain (GeedotWidget *);
int gdot_draw_radio_button_highlighted (GeedotWidget *);
int gdot_draw_radio_button_selected (GeedotWidget *);
int gdot_draw_scrollbar_plain (GeedotWidget *);
int gdot_draw_scrollbar_depressed (GeedotWidget *);
int gdot_draw_combobox_button_depressed (GeedotWidget *);
int gdot_draw_combobox_button_plain (GeedotWidget *);

int gdot_redraw_widget (GeedotWidget *);
int gdot_redraw_widget_recursive (GeedotWidget *);

int gdot_widget_verify_focus (int, int);
int gdot_widget_verify_pointer_focus (int, int);
int gdot_widget_clear_focus (void);
int gdot_widget_grab_focus (GeedotWidget *);
int gdot_widget_steal_focus (GeedotWidget *);
int gdot_widget_grab_motion_focus (GeedotWidget *);
int gdot_widget_steal_motion_focus (GeedotWidget *focus);
int gdot_widget_grab_depression_focus (GeedotWidget *focus);
int gdot_widget_set_active (GeedotWidget *, Bool);
int gdot_widget_set_visible (GeedotWidget *, Bool);
int gdot_widget_run_actions_for_event (GeedotWidget *, XEvent);
int gdot_widget_run_actions_recursive (GeedotWidget *, XEvent);
int gdot_widget_destroy (GeedotWidget *);
int gdot_widget_destroy_recursive (GeedotWidget *);
int gdot_widget_inherit (GeedotWidget *, GeedotWidget *);
int gdot_widget_reinherit_recursive (GeedotWidget *);
int gdot_widget_absolute_x (GeedotWidget *);
int gdot_widget_absolute_y (GeedotWidget *);
GeedotArea gdot_widget_absolute_area (GeedotWidget *);
int gdot_widget_is_parent_of_widget (GeedotWidget *, GeedotWidget *);
GeedotArea gdot_widget_fit (GeedotWidget *);
GeedotArea gdot_widget_shrinkwrap (GeedotWidget *);
GeedotWidget *gdot_toplevel_widget (GeedotWidget *);
GeedotWidget *gdot_focused_widget (void);
GeedotWidget *gdot_depression_focused_widget (void);
GeedotWidget *gdot_motion_focused_widget (void);
GeedotWidget *gdot_keyboard_focused_widget (void);
int gdot_area_bounds_location (GeedotArea, int, int);

int gdot_frame_handle_reconfiguration (GeedotWidget *, void *);
int gdot_frame_clear (GeedotWidget *);
int gdot_button_set_depressed (GeedotWidget *);
int gdot_button_set_released (GeedotWidget *);
int gdot_button_set_label (GeedotWidget *, char *);
int gdot_button_set_pixmap (GeedotWidget *, Pixmap);
int gdot_button_fit (GeedotWidget *);
int gdot_checkbox_set_depressed (GeedotWidget *);
int gdot_checkbox_set_selected (GeedotWidget *);
int gdot_checkbox_set_label (GeedotWidget *, char *);
int gdot_radio_button_set_selected (GeedotWidget *);
int gdot_radio_button_set_deselected (GeedotWidget *);
int gdot_radio_button_set_label (GeedotWidget *, char *);
int gdot_textbox_set_cursor_start (GeedotWidget *, void *);
int gdot_textbox_set_cursor_stop (GeedotWidget *, void *);
int gdot_textbox_set_label (GeedotWidget *, char *);
int gdot_textbox_set_text (GeedotWidget *, char *);
int gdot_textbox_append_text (GeedotWidget *, char *);
int gdot_textbox_grab_selection (GeedotWidget *, void *);
int gdot_textbox_set_selection (GeedotWidget *);
int gdot_textbox_handle_keypress (GeedotWidget *, void *);
int gdot_textbox_handle_keyrelease (GeedotWidget *, void *);
int gdot_textbox_collapse_selection (GeedotWidget *);
int gdot_scrollbar_set_grab_start (GeedotWidget *, void *);
int gdot_scrollbar_set_grab_stop (GeedotWidget *, void *);
int gdot_scrollbar_set_percentage (GeedotWidget *, float);
int gdot_scrollbar_move_bar (GeedotWidget *, int *);
int gdot_scrollbar_slide_bar (GeedotWidget *, void *);
int gdot_scrollbar_decrement_bar (GeedotWidget *, void *);
int gdot_scrollbar_increment_bar (GeedotWidget *, void *);
int gdot_combobox_expand (GeedotWidget *);
int gdot_combobox_set_depressed (GeedotWidget *);
int gdot_combobox_set_released (GeedotWidget *);
int gdot_combobox_check_click (GeedotWidget *, void *);
int gdot_combobox_set_option (GeedotWidget *);
GeedotWidget *gdot_combobox_get_option (GeedotWidget *);
char *gdot_combobox_get_option_string (GeedotWidget *);
int gdot_reset_pointer (GeedotWidget *);
int gdot_widget_has_focus (GeedotWidget *);
int gdot_redraw_frame (GeedotWidget *);
int gdot_redraw_exposed_frame (GeedotWidget *, void *);
int gdot_redraw_text (GeedotWidget *);
int gdot_redraw_label_text (GeedotWidget *);
int gdot_redraw_pixmap (GeedotWidget *);
int gdot_redraw_button (GeedotWidget *);
int gdot_redraw_checkbox (GeedotWidget *);
int gdot_redraw_radio_button (GeedotWidget *);
int gdot_redraw_textbox (GeedotWidget *);
int gdot_redraw_scrollbar (GeedotWidget *);
int gdot_redraw_combobox (GeedotWidget *);
int gdot_redraw_combobox_button (GeedotWidget *);
XEvent gdot_process_next_event (GeedotWidget *);
int gdot_get_event_type_mask (int);
int gdot_wait_for_event (GeedotWidget *);
int gdot_events_pending (GeedotWidget *);

void gdot_null_function (void);
